//
// Common macros & definitions
//

#ifndef R_VALUE
#define R_VALUE

#include <Rcpp.h>


//
//  GC Protected embodiment of R value
//
class RValue
{
  public:
    RValue (SEXP e)
      : _e(e), _count(new int(1))
    {
        if (e)
	    PROTECT(e);
    }

    RValue ()
      : _e(nullptr), _count(nullptr) { }

    RValue (const RValue& o)
      : _e(o._e), _count(o._count)
    {
        (*_count)++;
    }

    ~RValue()
    {
        remove();
    }

    // return underlying value
    operator SEXP () const
    {
        return _e;
    }

    // assignment 
    RValue& operator= (const RValue& o)
    {
        if (this != &o)
	{
	    remove();
	    _count = o._count;
	    (*_count)++;
	    _e = o._e;
	}

	return *this;
    }

  private:
  
    void remove ()
    {
        if (_count == nullptr)
	    return;
        if (--(*_count) > 0)
	    return;
       
	delete _count;
	if (!_e)
	    return;
	
	UNPROTECT(1);
    }
  
  private:
    SEXP    _e;
    int*    _count;
};

#endif
